%% Bode Plot Example
% This file generates a Bode plot for a first order system using the
% Matlab command "bode".

%% Transfer Function
b0 = 2;
a0 = 4;
G = tf(b0,[1 a0]);

%% Bode Plot

% Generate plot directly with Bode command
figure(1)
clf;
bode(G);

% Use Bode command to compute magnitude and phase data. Use this
% data to create own Bode plot.
% Note: The "bode" command returns the magnitude and phase in a 
% three-dimensional array with dimensions Ny-by-Nu-by-Nw where Ny/Nu/Nw
% are the number of outputs, inputs, and frequencies.  For single-input
% single-output systems this is 1-by-1-by-Nw.  The ":" operation, e.g.
% Gmag(:), converts this data into an Nw-by-1 column vector.
[Gmag,Gph_deg,w] = bode(G);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);

figure(2)
clf;

subplot(2,1,1)
semilogx(w,Gmag_dB,'b')
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
set(gca,'YTick',[-40 -20 -6 0],'LineWidth',1);

subplot(2,1,2)
semilogx(w,Gph_deg,'b')
ylim([-90 0]);
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
set(gca,'YTick',[-90 -45 0],'LineWidth',1);
grid on;

%% Highlight data points at certain frequencies
w = [2 4];         % Frequencies to simulate, rad/sec
Nw = numel(w);

[Gmag,Gph_deg,w] = bode(G,w);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);

subplot(2,1,1)
hold on;
semilogx(w,Gmag_dB,'ro')
hold off;

subplot(2,1,2)
hold on;
semilogx(w,Gph_deg,'ro')
hold off;
    
%% Evaluate G on a grid of frequencies 
wgrid = [0 1 2 3 4 6 10 20 inf]';
Ngrid = numel(wgrid);
Ggrid = zeros(Ngrid,1);
for i=1:Ngrid
    wi = wgrid(i);
    Ggrid(i) = evalfr(G,j*wi);
end

% Display magnitude (actual), magnitude (dB), phase (rad), and phase (deg)
Gmag = abs(Ggrid);
Gmag_dB = 20*log10(Gmag);
Gph = angle(Ggrid);
Gph_deg = (180/pi)*Gph;

[wgrid Gmag Gmag_dB Gph Gph_deg]


%% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure    
    
    figure(2)
    garyfyFigure    
    myprint(2,'BodeExample')
end
