%% Bode Plot: First-order pole with different phase plots
% This file generates a Bode plot for a first order system using the
% Matlab command "bode". Different phase plots are generated based
% on the sign of the ODE coefficients.

%% ODE Coefficients
%   dy/dt + a0*y(t) = b0*u(t)
b0 = 2;
a0 = 4;

%% Bode Plots for Different Signs of ODE Coefficients
% Note that the magnitude plot is identical for all four cases. Only the
% phase plot changes when the signs of the ODE coefficients are varied.

figure(1); clf;
co = get(groot,'defaultAxesColorOrder');
%set(gcf,'Position',[4 5 5.8 4.9]);
for i=1:4
    % Select signs for ODE coefficients
    if i==1
        G = tf(b0,[1 a0]);
    elseif i==2
        G = tf(-b0,[1 a0]);
    elseif i==3
        G = tf(b0,[1 -a0]);
    else
        G = tf(-b0,[1 -a0]);
    end
    
    % Use Bode command to compute exact magnitude and phase data.
    w = sort([a0 a0/10 10*a0 logspace(-2,3,200)]);
    [Gmag,Gph_deg,w] = bode(G,w);
    Gmag = Gmag(:);
    Gph_deg = Gph_deg(:);
    Gmag_dB = 20*log10(Gmag);
    G0 = b0/a0;
    G0dB = 20*log10(b0/a0);
    
    
    % Magnitude Plot
    subplot(2,1,1);
    semilogx(w/a0,Gmag_dB,'Color',co(i,:));
    if i==1
        hold on;
        ylabel('Magnitude (dB)');
        title('Bode Diagram');
        grid on;
        axis([1/100 100 -52 0]);
        set(gca,'LineWidth',2);
        set(gca,'YTick',[-46 -26 -9 -6],'YTickLabel',{'-46','-26','',''});
        text(0.61/100,-4,'-6','FontSize',14);
        text(0.61/100,-11,'-9','FontSize',14);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
    end
    
    % Phase Plot
    subplot(2,1,2);
    semilogx(w/a0,Gph_deg,'Color',co(i,:));
    if i==1
        hold on;
        axis([1/100 100 -180 180]);
        ylabel('Phase (deg)');
        xlabel('Frequency (rad/sec)');
        grid on;
        set(gca,'LineWidth',2);
        set(gca,'YTick',[-180 -135 -90 -45 0 45 90 135 180]);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
    end
end

subplot(2,1,1)
lh=legend(' 2/(s+4)','-2/(s+4)',' 2/(s-4)','-2/(s-4)',...
    'Location','SouthWest');

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'BodeFOPolePhase')
end

