%% Bode Plot: First-order zero with different phase plots
% This file generates a Bode plot for a first order zero using the
% Matlab command "bode". Different phase plots are generated based
% on the sign of the ODE coefficients.

%% ODE Coefficients
%   a0*y(t) = du/dt + b0*u
b0 = 2;
a0 = 4;

%% Bode Plots for Different Signs of ODE Coefficients
% Note that the magnitude plot is identical for all four cases. Only the
% phase plot changes when the signs of the ODE coefficients are varied.

figure(1); clf;
co = get(groot,'defaultAxesColorOrder');
%set(gcf,'Position',[4 5 5.8 4.9]);
for i=1:4
    % Select signs for ODE coefficients
    if i==1
        G = tf([1 b0],a0);
    elseif i==2
        G = tf([1 b0],-a0);
    elseif i==3
        G = tf([1 -b0],a0);
    else
        G = tf([1 -b0],-a0);
    end
    
    % Use Bode command to compute exact magnitude and phase data.
    w = sort([b0 b0/10 10*b0 logspace(-2,3,200)]);
    [Gmag,Gph_deg,w] = bode(G,w);
    Gmag = Gmag(:);
    Gph_deg = Gph_deg(:);
    if Gph_deg(end)>180
        Gph_deg = Gph_deg-360;
    end 
    Gmag_dB = 20*log10(Gmag);
    G0 = b0/a0;
    G0dB = 20*log10(b0/a0);    
    
    % Magnitude Plot
    subplot(2,1,1);
    semilogx(w/b0,Gmag_dB,'Color',co(i,:));
    if i==1
        hold on;
        ylabel('Magnitude (dB)');
        title('Bode Diagram');
        grid on;
        axis([1/100 100 -9 40]);
        set(gca,'LineWidth',2);
        set(gca,'YTick',[-6 -3 14 34],'YTickLabel',{'','','14','34'});
        text(0.61/100,-6,'-6','FontSize',14);
        text(0.61/100,0,'-3','FontSize',14);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.02', '0.2', '2', '20', '200'});
    end
    
    % Phase Plot
    subplot(2,1,2);
    semilogx(w/b0,Gph_deg,'Color',co(i,:));
    if i==1
        hold on;
        axis([1/100 100 -180 180]);
        ylabel('Phase (deg)');
        xlabel('Frequency (rad/sec)');
        grid on;
        set(gca,'LineWidth',2);
        set(gca,'YTick',[-180 -135 -90 -45 0 45 90 135 180]);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.02', '0.2', '2', '20', '200'});
    end
end

subplot(2,1,1)
lh=legend(' (s+2)/4','-(s+2)/4',' (s-2)/4','-(s-2)/4',...
    'Location','NorthWest');

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'BodeFOZeroPhase')
end

