%% Bode Plot: Second order Underdamped System (pole)
% This file generates a Bode plot for a second order underdamped system
% using the Matlab command "bode".

%% Transfer Function Data
% Transfer function is G(s) = b0/(s^2+2*z*wn*s+wn^2)
% Several damping ratios are selected.
b0 = 16;
wn = 4;
z = [0.7 0.05];

%% Exact Bode Plot
figure(1); clf;
for i=1:2
    % Use Bode command to compute exact magnitude and phase data.
    G = tf(b0,[1 2*z(i)*wn wn^2]);
    
    w = sort([wn wn/10 10*wn logspace(-2,3,200)]);
    [Gmag,Gph_deg,w] = bode(G,w);
    Gmag = Gmag(:);
    Gph_deg = Gph_deg(:);
    Gmag_dB = 20*log10(Gmag);
    G0 = b0/wn^2;
    G0dB = 20*log10(G0);
        
    % Magnitude Plot
    subplot(2,1,1);
    if i==1
        Gph_deg1 = Gph_deg;
        Gmag_dB1 = Gmag_dB;

        semilogx(w/wn,Gmag_dB,'k-.');
        hold on;
    else
        Gph_deg2 = Gph_deg;
        Gmag_dB2 = Gmag_dB;
        
        semilogx(w/wn,Gmag_dB,'b');
        hold off;
        legend(['\zeta = ' num2str(z(1),'%0.2f')],...
            ['\zeta = ' num2str(z(2),'%0.2f')],'Location','SouthWest');
    end        
    if i==1
        ylabel('Magnitude (dB)');
        title('Bode Diagram');
        grid on;        
        axis([1/100 100 -90 28]);
        set(gca,'LineWidth',2);
        set(gca,'YTick',-80:20:20);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
    end
    
    % Phase Plot
    subplot(2,1,2);
    if i==1
        semilogx(w/wn,Gph_deg,'k-.');
        hold on;
    else
        semilogx(w/wn,Gph_deg,'b');
        hold off;        
    end
    if i==1
        axis([1/100 100 -180 0]);
        ylabel('Phase (deg)');
        xlabel('Frequency (rad/sec)');
        grid on;
        set(gca,'LineWidth',2);
        set(gca,'YTick',[-180 -135 -90 -45 0]);
        set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
            'XtickLabel',{'0.04', '0.4', '4', '40', '400'});
    end
end

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'BodeSOPole1')
end


%% Straight-line (asymptotic) Bode approximation

% Normalize frequency by corner frequency
w = w/wn;

% Frequencies for straight-line approximations
wlow = w( w<= 1/10);
wmid1 = w( w>= 1/10 & w<=1 );
wmid2 = w( w>= 1 & w<=10 );
whigh = w( w>= 10);
Nw = [numel(wlow) numel(wmid1) numel(wmid2) numel(whigh)];

figure(2);
clf;

% Magnitude Plots
subplot(2,1,1);
ph(1) = semilogx(wlow, G0dB*ones(Nw(1),1),'r');
hold on;
ph(2) = semilogx(wmid1, G0dB*ones(Nw(2),1),'r');
ph(3) = semilogx(whigh, 40*log10(G0./whigh),'r');
ph(4) = semilogx(wmid2, 40*log10(G0./wmid2),'r');
ph(5:6) = semilogx(w,Gmag_dB1,'k:',w,Gmag_dB2,'b:');
ylabel('Magnitude (dB)');
title('Bode Diagram');
grid on;
axis([1/100 100 G0dB-90 G0dB+28]);
set(gca,'LineWidth',2);
set(gca,'YTick',G0dB-[80 60 40 20 0 -20],...
    'YtickLabel',{'','','   ',''});
th(1) = text(0.71/100,0,'g','FontSize',14);
th(2) = text(1.25/100,17,'g+20','FontSize',14);
th(4) = text(1.25/100,-38,'g-40','FontSize',14);
th(3) = text(1.25/100,-78,'g-80','FontSize',14);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'\omega_n/100', '\omega_n/10', '\omega_n', ...
    '10\omega_n', '100\omega_n'});
hold off;

% Phase Plot
subplot(2,1,2);
ph(7) = semilogx(wlow, 0*ones(Nw(1),1),'r');
hold on;
wmid = [wmid1; wmid2];
fac = log10(wmid/wmid(1)) / log10(wmid(end)/wmid(1));
ph(8) = semilogx(wmid, -180*fac,'r');
ph(9) = semilogx(whigh, -180*ones(Nw(4),1),'r');
ph(10:11)=semilogx(w,Gph_deg1,'k:',w,Gph_deg2,'b:');
axis([1/100 100 -180 0]);
legend(ph([7 10 11]),'Approx',...
    ['Actual: \zeta = ' num2str(z(1),'%0.2f')],...
    ['Actual: \zeta = ' num2str(z(2),'%0.2f')],...
    'Location','SouthWest');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
grid on;
set(gca,'LineWidth',2);
set(gca,'YTick',[-180 -135 -90 -45 0]);
set(gca,'Xtick',[1/100 1/10 1 10 100],'XMinorGrid','off', ...
    'XtickLabel',{'\omega_n/100', '\omega_n/10', '\omega_n', ...
    '10\omega_n', '100\omega_n'});
hold off;

% Save Plot
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(2)
    garyfyFigure
    set(ph([1:4 7:9]),'LineWidth',2)
    myprint(2,'BodeSOPole2')
end
