%% Overdamped Second Order System
% This file generates the Bode plot for an overdamped second order
% system.  The system transfer function can be written as 
%   G(s)= b0/(s-p1)/(s-p2)  where p1 and p2 are the real poles
% The Bode plot of G(s) can be constructed from Bode plots of b0/(s-p1)
% and 1/(s-p2).


%% System
w1 = 0.2;             % Pole of first system is at -w1
G1 = tf(1,[1 w1]);

w2 = 1;           % Pole of first system is at -w2
G2 = tf(0.5,[1 w2]);

G = G1*G2;

%% Straight-line (asymptotic) Bode approximation
Nw = 200;
w = logspace(-2,log10(20),Nw);

G1DC = dcgain(G1);
G1DCdB = 20*log10(G1DC);
G1dB = zeros(Nw,1);
G1dB( w <= w1 ) = G1DCdB;
G1dB( w > w1 ) = 20*log10(G1DC*w1./ w(w>w1) );

G1ph = zeros(Nw,1);
G1ph( w<=w1/10 ) = 0;
G1ph( w>=10*w1 ) = -90;
wmid = w(w>w1/10 & w<10*w1);
fac = log10(wmid/wmid(1)) / log10(wmid(end)/wmid(1));
G1ph( w>w1/10 & w<10*w1 ) = -90*fac;

G2DC = dcgain(G2);
G2DCdB = 20*log10(G2DC);
G2dB = zeros(Nw,1);
G2dB( w <= w2 ) = G2DCdB;
G2dB( w > w2 ) = 20*log10(G2DC*w2./ w(w>w2) );

G2ph = zeros(Nw,1);
G2ph( w<=w2/10 ) = 0;
G2ph( w>=10*w2 ) = -90;
wmid = w(w>w2/10 & w<10*w2);
fac = log10(wmid/wmid(1)) / log10(wmid(end)/wmid(1));
G2ph( w>w2/10 & w<10*w2 ) = -90*fac;


figure(1);
clf

% Magnitude Plot
subplot(2,1,1);
semilogx(w,G1dB+G2dB,'b',w,G1dB,'r:',w,G2dB,'g-.');
ylabel('Magnitude (dB)');
title('Bode Diagram');
axis([w(1) w(end) -40 +24])
grid on;
set(gca,'LineWidth',2);
legend('G','G1','G2','Location','Best');
set(gca,'YTick',[-40 -20 -6 8 14],...
    'YTickLabel',{'-40', '-20', '-6', '', '14'});
th = text(0.0075,6,'8');

% Phase Plot
subplot(2,1,2);
semilogx(w,G1ph+G2ph,'b',w,G1ph,'r:',w,G2ph,'g-.');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
axis([w(1) w(end) -180 0])
grid on;
set(gca,'LineWidth',2);

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    set(th,'FontSize',14)
    myprint(1,'BodeOD1')
end

%% Exact Bode plot and approximation
[Gmag,Gph_deg,w] = bode(G,w);
Gmag = Gmag(:);
Gph_deg = Gph_deg(:);
Gmag_dB = 20*log10(Gmag);

figure(2);
clf

% Magnitude Plot
subplot(2,1,1);
semilogx(w,G1dB+G2dB,'b',w,Gmag_dB,'k-.');
ylabel('Magnitude (dB)');
title('Bode Diagram');
axis([w(1) w(end) -40 +24])
grid on;
set(gca,'LineWidth',2);
set(gca,'YTick',[-40 -20 -6 8 14],...
    'YTickLabel',{'-40', '-20', '-6', '', '14'});
th = text(0.0075,6,'8');

% Phase Plot
subplot(2,1,2);
semilogx(w,G1ph+G2ph,'b',w,Gph_deg,'k-.');
ylabel('Phase (deg)');
xlabel('Frequency (rad/sec)');
axis([w(1) w(end) -180 0])
grid on;
set(gca,'LineWidth',2);
legend('G Approx.','G Exact','Location','NorthEast');

% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(2)
    garyfyFigure
    set(th,'FontSize',14)
    myprint(2,'BodeOD2')
end
